@@ -465,6 +465,20 @@ static char *opcode2str(uint16_t opcode)
 	return cmd;
 }
 
+static char *linktype2str(uint8_t type)
+{
+	switch (type) {
+	case 0x00:
+		return "SCO";
+	case 0x01:
+		return "ACL";
+	case 0x02:
+		return "eSCO";
+	default:
+		return "Unknown";
+	}
+}
+
 static char *role2str(uint8_t role)
 {
 	switch (role) {
@@ -962,6 +976,22 @@ static inline void write_link_policy_dump(int level, struct frame *frm)
 	}
 }
 
+static inline void write_default_link_policy_dump(int level, struct frame *frm)
+{
+	uint16_t policy = btohs(htons(get_u16(frm)));
+	char *str;
+
+	p_indent(level, frm);
+	printf("policy 0x%2.2x\n", policy);
+
+	str = hci_lptostr(policy);
+	if (str) {
+		p_indent(level, frm);
+		printf("Link policy: %s\n", str);
+		free(str);
+	}
+}
+
 static inline void sniff_subrating_dump(int level, struct frame *frm)
 {
 	sniff_subrating_cp *cp = frm->ptr;
@@ -1433,6 +1463,8 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_READ_LINK_POLICY:
 			generic_command_dump(level + 1, frm);
 			return;
+		case OCF_READ_DEFAULT_LINK_POLICY:
+			return;
 		case OCF_SWITCH_ROLE:
 			accept_conn_req_dump(level + 1, frm);
 			return;
@@ -1442,6 +1474,9 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_WRITE_LINK_POLICY:
 			write_link_policy_dump(level + 1, frm);
 			return;
+		case OCF_WRITE_DEFAULT_LINK_POLICY:
+			write_default_link_policy_dump(level + 1, frm);
+			return;
 		case OCF_SNIFF_SUBRATING:
 			sniff_subrating_dump(level + 1, frm);
 			return;
@@ -1653,6 +1688,51 @@ static inline void status_mode_dump(int level, struct frame *frm)
 	}
 }
 
+static inline void read_link_policy_dump(int level, struct frame *frm)
+{
+	read_link_policy_rp *rp = frm->ptr;
+	uint16_t policy = btohs(rp->policy);
+	char *str;
+
+	p_indent(level, frm);
+	printf("status 0x%2.2x handle %d policy 0x%2.2x\n",
+				rp->status, btohs(rp->handle), policy);
+
+	if (rp->status > 0) {
+		p_indent(level, frm);
+		printf("Error: %s\n", status2str(rp->status));
+	} else {
+		str = hci_lptostr(policy);
+		if (str) {
+			p_indent(level, frm);
+			printf("Link policy: %s\n", str);
+			free(str);
+		}
+	}
+}
+
+static inline void read_default_link_policy_dump(int level, struct frame *frm)
+{
+	uint8_t status = get_u8(frm);
+	uint16_t policy = btohs(htons(get_u16(frm)));
+	char *str;
+
+	p_indent(level, frm);
+	printf("status 0x%2.2x policy 0x%2.2x\n", status, policy);
+
+	if (status > 0) {
+		p_indent(level, frm);
+		printf("Error: %s\n", status2str(status));
+	} else {
+		str = hci_lptostr(policy);
+		if (str) {
+			p_indent(level, frm);
+			printf("Link policy: %s\n", str);
+			free(str);
+		}
+	}
+}
+
 static inline void read_pin_type_dump(int level, struct frame *frm)
 {
 	read_pin_type_rp *rp = frm->ptr;
@@ -1946,7 +2026,7 @@ static inline void read_local_oob_data_dump(int level, struct frame *frm)
 static inline void read_local_version_dump(int level, struct frame *frm)
 {
 	read_local_version_rp *rp = frm->ptr;
-	uint16_t manufacturer = btohs(rp->manufacturer);
+	uint16_t manufacturer = btohs(rp->err);
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x\n", rp->status);
@@ -2174,10 +2254,19 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 
 	case OGF_LINK_POLICY:
 		switch (ocf) {
+		case OCF_READ_LINK_POLICY:
+			read_link_policy_dump(level, frm);
+			return;
 		case OCF_WRITE_LINK_POLICY:
 		case OCF_SNIFF_SUBRATING:
 			generic_response_dump(level, frm);
 			return;
+		case OCF_READ_DEFAULT_LINK_POLICY:
+			read_default_link_policy_dump(level, frm);
+			return;
+		case OCF_WRITE_DEFAULT_LINK_POLICY:
+			status_response_dump(level, frm);
+			return;
 		}
 		break;
 
@@ -2405,7 +2494,7 @@ static inline void conn_complete_dump(int level, struct frame *frm)
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d bdaddr %s type %s encrypt 0x%2.2x\n",
 			evt->status, btohs(evt->handle), addr,
-			evt->link_type == 1 ? "ACL" : "SCO", evt->encr_mode);
+			linktype2str(evt->link_type), evt->encr_mode);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -2423,7 +2512,7 @@ static inline void conn_request_dump(int level, struct frame *frm)
 	p_indent(level, frm);
 	printf("bdaddr %s class 0x%2.2x%2.2x%2.2x type %s\n",
 			addr, evt->dev_class[2], evt->dev_class[1],
-			evt->dev_class[0], evt->link_type == 1 ? "ACL" : "SCO");
+			evt->dev_class[0], linktype2str(evt->link_type));
 }
 
 static inline void disconn_complete_dump(int level, struct frame *frm)
@@ -2518,7 +2607,7 @@ static inline void read_remote_features_complete_dump(int level, struct frame *f
 static inline void read_remote_version_complete_dump(int level, struct frame *frm)
 {
 	evt_read_remote_version_complete *evt = frm->ptr;
-	uint16_t manufacturer = btohs(evt->manufacturer);
+	uint16_t manufacturer = btohs(evt->err);
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d\n", evt->status, btohs(evt->handle));
@@ -2641,7 +2730,7 @@ static inline void data_buffer_overflow_dump(int level, struct frame *frm)
 	evt_data_buffer_overflow *evt = frm->ptr;
 
 	p_indent(level, frm);
-	printf("type %s\n", evt->link_type == 1 ? "ACL" : "SCO");
+	printf("type %s\n", linktype2str(evt->link_type));
 }
 
 static inline void read_clock_offset_complete_dump(int level, struct frame *frm)
@@ -2957,7 +3046,7 @@ static inline void event_dump(int level, struct frame *frm)
 		evt_cmd_complete *cc = frm->ptr;
 		if (cc->opcode == cmd_opcode_pack(OGF_INFO_PARAM, OCF_READ_LOCAL_VERSION)) {
 			read_local_version_rp *rp = frm->ptr + EVT_CMD_COMPLETE_SIZE;
-			manufacturer = rp->manufacturer;
+			manufacturer = rp->err;
 		}
 	}
 
