@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2006  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2007  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -420,7 +420,7 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	if (ver.manufacturer != 12) {
+	if (ver.err != 12) {
 		fprintf(stderr, "Can't find sniffer at hci%d: %s (%d)\n",
 						dev, strerror(ENOSYS), ENOSYS);
 		hci_close_dev(dd);
